@@ -78,6 +78,7 @@ typedef struct mapcache_image_format_mixed mapcache_image_format_mixed;
 typedef struct mapcache_image_format_png mapcache_image_format_png;
 typedef struct mapcache_image_format_png_q mapcache_image_format_png_q;
 typedef struct mapcache_image_format_jpeg mapcache_image_format_jpeg;
+typedef struct mapcache_image_format_raw mapcache_image_format_raw;
 typedef struct mapcache_cfg mapcache_cfg;
 typedef struct mapcache_tileset mapcache_tileset;
 typedef struct mapcache_cache mapcache_cache;
@@ -115,9 +116,9 @@ typedef struct mapcache_locker mapcache_locker;
 typedef enum {
   MAPCACHE_DIMENSION_VALUES,
   MAPCACHE_DIMENSION_REGEX,
-  MAPCACHE_DIMENSION_INTERVALS,
-  MAPCACHE_DIMENSION_TIME,
-  MAPCACHE_DIMENSION_SQLITE
+  MAPCACHE_DIMENSION_POSTGRESQL,
+  MAPCACHE_DIMENSION_SQLITE,
+  MAPCACHE_DIMENSION_ELASTICSEARCH
 } mapcache_dimension_type;
 
 typedef enum {
@@ -663,7 +664,7 @@ MS_DLL_EXPORT int mapcache_config_services_enabled(mapcache_context *ctx, mapcac
 /** @{ */
 
 typedef enum {
-  GC_UNKNOWN, GC_PNG, GC_JPEG
+  GC_UNKNOWN, GC_PNG, GC_JPEG, GC_RAW
 } mapcache_image_format_type;
 
 typedef enum {
@@ -1325,8 +1326,14 @@ MS_DLL_EXPORT int mapcache_util_extract_int_list(mapcache_context *ctx, const ch
                                    int *numbers_count);
 MS_DLL_EXPORT int mapcache_util_extract_double_list(mapcache_context *ctx, const char* args, const char *sep, double **numbers,
                                       int *numbers_count);
-char *mapcache_util_str_replace(apr_pool_t *pool, const char *string, const char *substr,
+MS_DLL_EXPORT char *mapcache_util_str_replace(apr_pool_t *pool, const char *string, const char *substr,
                                 const char *replacement );
+char *mapcache_util_dbl_replace(apr_pool_t *pool, const char *string, const char *substr,
+                                double replacement );
+char *mapcache_util_str_replace_all(apr_pool_t *pool, const char *string, const char *substr,
+                                    const char *replacement );
+char *mapcache_util_dbl_replace_all(apr_pool_t *pool, const char *string, const char *substr,
+                                    double replacement );
 void mapcache_util_quadkey_decode(mapcache_context *ctx, const char *quadkey, int *x, int *y, int *z);
 
 char* mapcache_util_quadkey_encode(mapcache_context *ctx, int x, int y, int z);
@@ -1339,7 +1346,7 @@ char* mapcache_util_quadkey_encode(mapcache_context *ctx, int x, int y, int z);
  * \return the original string if no matches were found, or the sanitized
  *         string allocated from the given pool
  */
-char* mapcache_util_str_sanitize(apr_pool_t *pool, const char *str, const char* from, char to);
+MS_DLL_EXPORT char* mapcache_util_str_sanitize(apr_pool_t *pool, const char *str, const char* from, char to);
 
 typedef enum {
   MAPCACHE_UTIL_XML_SECTION_TEXT,
@@ -1349,7 +1356,7 @@ typedef enum {
 
 char* mapcache_util_str_xml_escape(apr_pool_t *pool, const char *str, mapcache_util_xml_section_type xml_section_type);
 
-char* mapcache_util_get_tile_dimkey(mapcache_context *ctx, mapcache_tile *tile, char* sanitized_chars, char *sanitize_to);
+MS_DLL_EXPORT char* mapcache_util_get_tile_dimkey(mapcache_context *ctx, mapcache_tile *tile, char* sanitized_chars, char *sanitize_to);
 
 char* mapcache_util_get_tile_key(mapcache_context *ctx, mapcache_tile *tile, char *stemplate,
                                  char* sanitized_chars, char *sanitize_to);
@@ -1428,10 +1435,16 @@ struct mapcache_image_format_mixed {
 
 mapcache_buffer* mapcache_empty_png_decode(mapcache_context *ctx, int width, int height, const unsigned char *hex_color, int *is_empty);
 
-
 mapcache_image_format* mapcache_imageio_create_mixed_format(apr_pool_t *pool,
     char *name, mapcache_image_format *transparent, mapcache_image_format *opaque, unsigned int alpha_cutoff);
 
+struct mapcache_image_format_raw {
+  mapcache_image_format format;
+};
+
+mapcache_image_format* mapcache_imageio_create_raw_format(apr_pool_t *pool, char *name, char *extension, char *mime_type); 
+int mapcache_imageio_is_raw_tileset(mapcache_tileset *tileset);
+
 /**\class mapcache_image_format_png_q
  * \brief Quantized PNG format
  * \extends mapcache_image_format_png
@@ -1556,12 +1569,13 @@ void mapcache_tile_set_cached_dimension(mapcache_context *ctx, mapcache_tile *ti
 void mapcache_map_set_cached_dimension(mapcache_context *ctx, mapcache_map *map, const char *name, const char *value);
 void mapcache_tile_set_requested_dimension(mapcache_context *ctx, mapcache_tile *tile, const char *name, const char *value);
 void mapcache_map_set_requested_dimension(mapcache_context *ctx, mapcache_map *map, const char *name, const char *value);
-void mapcache_set_requested_dimension(mapcache_context *ctx, apr_array_header_t *dimensions, const char *name, const char *value);
-void mapcache_set_cached_dimension(mapcache_context *ctx, apr_array_header_t *dimensions, const char *name, const char *value);
+MS_DLL_EXPORT void mapcache_set_requested_dimension(mapcache_context *ctx, apr_array_header_t *dimensions, const char *name, const char *value);
+MS_DLL_EXPORT void mapcache_set_cached_dimension(mapcache_context *ctx, apr_array_header_t *dimensions, const char *name, const char *value);
 MS_DLL_EXPORT apr_array_header_t *mapcache_requested_dimensions_clone(apr_pool_t *pool, apr_array_header_t *src);
 
 struct mapcache_dimension {
   mapcache_dimension_type type;
+  int isTime;
   char *name;
   char *unit;
   apr_table_t *metadata;
@@ -1570,7 +1584,14 @@ struct mapcache_dimension {
   /**
    * \brief return the list of dimension values that match the requested entry
    */
-  apr_array_header_t* (*get_entries_for_value)(mapcache_context *ctx, mapcache_dimension *dimension, const char *value,
+  apr_array_header_t* (*_get_entries_for_value)(mapcache_context *ctx, mapcache_dimension *dimension, const char *value,
+                       mapcache_tileset *tileset, mapcache_extent *extent, mapcache_grid *grid);
+
+  /**
+   * \brief return the list of dimension values that match the requested time range
+   */
+  apr_array_header_t* (*_get_entries_for_time_range)(mapcache_context *ctx, mapcache_dimension *dimension, const char *value,
+                       time_t start, time_t end,
                        mapcache_tileset *tileset, mapcache_extent *extent, mapcache_grid *grid);
 
   /**
@@ -1593,8 +1614,15 @@ struct mapcache_dimension {
 
 mapcache_dimension* mapcache_dimension_values_create(mapcache_context *ctx, apr_pool_t *pool);
 mapcache_dimension* mapcache_dimension_sqlite_create(mapcache_context *ctx, apr_pool_t *pool);
+mapcache_dimension* mapcache_dimension_postgresql_create(mapcache_context *ctx, apr_pool_t *pool);
 mapcache_dimension* mapcache_dimension_regex_create(mapcache_context *ctx, apr_pool_t *pool);
 mapcache_dimension* mapcache_dimension_time_create(mapcache_context *ctx, apr_pool_t *pool);
+mapcache_dimension* mapcache_dimension_elasticsearch_create(mapcache_context *ctx, apr_pool_t *pool);
+
+MS_DLL_EXPORT apr_array_header_t* mapcache_dimension_get_entries_for_value(mapcache_context *ctx, mapcache_dimension *dimension, const char *value,
+                       mapcache_tileset *tileset, mapcache_extent *extent, mapcache_grid *grid);
+apr_array_header_t* mapcache_dimension_time_get_entries_for_value(mapcache_context *ctx, mapcache_dimension *dimension, const char *value,
+                       mapcache_tileset *tileset, mapcache_extent *extent, mapcache_grid *grid);
 
 int mapcache_is_axis_inverted(const char *srs);
 
